        namespace { const char file_array[] = { '\x72','\x63','\x3a','\x3a','\x63','\x61','\x6c','\x3a','\x3a','\x74','\x65','\x73','\x74','\x5f','\x64','\x61','\x74','\x61', 0 }; }
        namespace cmrc { namespace rc { namespace res_chars {
        extern const char* const f_a280_cal_test_cal_begin = file_array;
        extern const char* const f_a280_cal_test_cal_end = file_array + 18;
        }}}
    